'use strict';

// //////////////////////////////////////////////////////////////////////////////
// Préliminaire : un petit glossaire des objets et fonctions utiles
// //////////////////////////////////////////////////////////////////////////////

/*
  Structures javascripts
  ======================
  Voir https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide

  déclaration de variables : let et const
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Types_et_grammaire#D.C3.A9clarations

  déclaration de fonctions et arrows
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Fonctions
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Fonctions#Fonctions_fl.C3.A9ch.C3.A9es

  comparaisons == et ===
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Expressions_et_Op%C3%A9rateurs#Op%C3%A9rateurs_de_comparaison

  différents boucles for
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Boucles_et_it%C3%A9ration

    * for (let i = 0; i < max; i += 1) {...}
    * for (let e of es) {...}

  les objets : { prop1 : valeur1, prop2 : valeur2 }
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Utiliser_les_objets#Les_objets_et_les_propri.C3.A9t.C3.A9s

  les chaines et les literaux de gabarit : "chaine", 'chaine' et `chaine ${expr}`
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Formatage_du_texte#Les_litt%C3%A9raux_de_mod%C3%A8le_multi-lignes

  Tableaux : Array
  ================
  Voir https://developer.mozilla.org/fr/docs/Web/JavaScript/Guide/Collections_index%C3%A9es

  * let tab = [42, 43];
  * tri : méthode .sort(comp)
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/sort
  * concatenation
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/concat
  * ajout d'élément : méthode .push()
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/push
  * boucle forEach
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/forEach
  * transformation : méthode .map()
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/map
  * filtrage : méthode .filter()
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/filter
  * transformer en tableau : Array.from
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Array/from

  API DOM pour manipuler la page web
  ==================================

  * Contenu HTML en lecture et écriture : .innerHTML
    https://developer.mozilla.org/fr/docs/Web/API/Element/innertHTML
  * Sélection d'un noeud par identifiant : document.getElementById
    https://developer.mozilla.org/fr/docs/Web/API/Document/getElementById
  * Sélection de *tous les noeuds* avec un sélecteur CSS : document.querySelectorAll(css)
    https://developer.mozilla.org/fr/docs/Web/API/Document/querySelectorAll
    https://developer.mozilla.org/fr/docs/Web/CSS/S%C3%A9lecteurs_CSS
  * Créer un élément HTML : document.createElement(balise)
    https://developer.mozilla.org/fr/docs/Web/API/Document/createElement
  * Ajouter un élément à la page : .appendChild()
    https://developer.mozilla.org/fr/docs/Web/API/Node/appendChild
  * Ajoute un écouteur sur un élément : .addEventListener(evt, handler)
    https://developer.mozilla.org/fr/docs/Web/API/EventTarget/addEventListener
    https://developer.mozilla.org/en-US/docs/Web/API/Document/DOMContentLoaded_event
    https://developer.mozilla.org/en-US/docs/Web/API/Element/click_event

  Divers
  ======

  * Conversion de chaines en entier
  * Test si nombre entier : Number.isInteger()
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Number/isInteger
  * Test si nombre : Number.isNaN()
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Number/isNaN
  * Popup pour saisie : prompt()
    https://developer.mozilla.org/fr/docs/Web/API/Window/prompt
  * Les objets Date
    https://developer.mozilla.org/fr/docs/Web/JavaScript/Reference/Objets_globaux/Date

*/

// //////////////////////////////////////////////////////////////////////////////
// Exercice : bases du langage et utilisation de la console
// //////////////////////////////////////////////////////////////////////////////

function fibonacci(n) {
  if (!Number.isInteger(n) || n < 0) {
    return undefined;
  }
  if (n === 0 || n === 1) {
    return n;
  }

  return (fibonacci(n - 1) + fibonacci(n - 2));
}


// range //////////////////////////////////////////////////////////////////////////////

// Fonction range, pas trop de choix ici dans l'écriture
function range(stop, start = 0, step = 1) {
  const res = [];

  // je mets une "garde" ici pour éviter des boucles sans fin si step est nul ou négatif
  if (!Number.isInteger(step) || step < 1) return res;

  // for "à l'ancienne" du C ou java. On ne peut pas faire bien mieux
  for (let i = start; i < stop; i += step) {
    res.push(i);
  }
  return res;
}

// //////////////////////////////////////////////////////////////////////////////
// Exercice : events pour range
// //////////////////////////////////////////////////////////////////////////////


function handler_exercice_range() {
  /* Exercice : fonction range */
  // La balise qui va contenir le résultat
  const output = document.getElementById('output');
  // On va cherche les valeurs saisies et on les transforme en entier en base 10
  let stop_val = Number.parseInt(document.getElementById('input-range-stop').value, 10);
  let start_val = Number.parseInt(document.getElementById('input-range-start').value, 10);
  let step_val = Number.parseInt(document.getElementById('input-range-step').value, 10);

  // console.log(stop_val, start_val, step_val);
  // Si la chaine lue n'est pas convertible en nombre, parseInt génère un NaN,
  // Je vérifie les valeur pour éviter les problèmes.
  if (Number.isNaN(stop_val)) {
    stop_val = 0;
  }
  if (Number.isNaN(start_val)) {
    start_val = 0;
  }
  if (Number.isNaN(step_val)) {
    step_val = 1;
  }
  // console.log(stop_val, start_val, step_val);

  const tab = range(stop_val, start_val, step_val);
  // on ajoute le résultat en convertissant le tableau en chaine
  output.innerHTML += `<p>${JSON.stringify(tab)}</p>`;

  // Alternative
  // génération d'un code html qui contient la liste à afficher à partir du tableau
  // En gros :
  // <ul>
  //  <li>0</li>
  //  <li>1</li>
  //  ...
  // </ul>
  // les \n et \t ne sont la que pour faire joli
  let code = '<ul>\n';

  for (const elem of tab) {
    code += `\t<li>${elem}</li>\n`;
  }
  code += '</ul>\n';
  // output.innerHTML = code;
}

// Associer les menus pour appeller la fonction range///////////////////////////
function handler_reset_range() {
  const output = document.getElementById('output');

  // on récupère des REFERENCES vers les 3 objets du DOM correspondant aux inputs
  const stop = document.getElementById('input-range-stop');
  const start = document.getElementById('input-range-start');
  const step = document.getElementById('input-range-step');

  stop.value = 10;
  start.value = '';
  step.value = '';
  output.innerHTML = '';

  // /!\ le code suivant ne permet PAS de modifier la valeur !
  // car ".value" étant atomique, il est copié par valeur...
  // const stop = document.getElementById('input-range-stop').value;
  // stop = 10;
}


// //////////////////////////////////////////////////////////////////////////////
// Exercice : Konami code
// //////////////////////////////////////////////////////////////////////////////



// eslint-disable-next-line no-unused-vars
const follow_mouse = (function () {
  // savoir si on a déjà le tracker
  let toggle=false;

  // une variable privée à la fonction pour se rappeller de la position
  const mouse_pos = { x: 0, y: 0 };

  // on ajoute un petit disque gris au div avec la classe dot qu'on trouve dans la css
  const circle = document.createElement('div');
  circle.classList.add('dot');
  const circle_elt = document.body.appendChild(circle);
  circle.classList.toggle("invisible");
  
  // le timer
  let intervalID;

  function f() {
    toggle = !toggle;

    if(toggle){
      circle.classList.toggle("invisible");

      // quand la souris bouge, on met à jour
      document.onmousemove = (e) => {
        mouse_pos.x = e.pageX;
        mouse_pos.y = e.pageY;
      };

      // toutes les 50ms, on déplace notre <div> qui est rendu par le disque gris
      intervalID = setInterval(() => {
        circle_elt.style.left = `${mouse_pos.x - 25}px`;
        circle_elt.style.top = `${mouse_pos.y - 25}px`;
      }, 50);
    }
    else{
      clearInterval(intervalID);
      document.onmousemove = undefined;
      circle.classList.toggle("invisible");
    }
  }
  return f;
})();


// //////////////////////////////////////////////////////////////////////////////
// Exercice : Konami code
// //////////////////////////////////////////////////////////////////////////////


//Haut, haut, bas, bas, gauche, droite, gauche, droite, B, A
const code = ['arrowup', 'arrowup', 'arrowdown', 'arrowdown', 'arrowleft', 'arrowright', 'arrowleft', 'arrowright', 'b', 'a'];
const konami = (function () {
  let n = 0;
  return function(event){
    // console.info(`konami: event.keyCode=${event.key}, n=${n}`);
    if (event.key.toLowerCase() === code[n++]) {
      if (n === code.length) {
        alert('Konami !!!'); // à remplacer par votre code
        n = 0;
      }
    }
    else
      n = 0;
  }
})();

// //////////////////////////////////////////////////////////////////////////////
// Exercice : Téléchargement asynchrone
// //////////////////////////////////////////////////////////////////////////////


function download_json(callback) {
  const url = 'https://perso.liris.cnrs.fr/romuald.thion/files/Enseignement/DIU-EIL/DIU-EIL-WEB.json';

  fetch(url)
    .then(response => response.json())
    .then(callback)
    .catch(console.error);
}

function replace_btn() {
  this.innerText = prompt('Saisir une valeur');
} 

// //////////////////////////////////////////////////////////////////////////////
document.addEventListener('DOMContentLoaded', () => {
  document.getElementById('btn-exec').onclick = replace_btn;    
  document.onkeydown = konami;
  document.getElementById('btn-range-reset').onclick = handler_reset_range;
  document.getElementById('btn-range-eval').onclick= handler_exercice_range;
  document.getElementById('btn-mouse').onclick = follow_mouse;
}, false);
